/*!\file: CreateParametersPrognostic2.cpp
 * \brief driver for creating parameters dataset, for prognostic analysis.
 */ 


#include "../../DataSet/DataSet.h"
#include "../../toolkits/toolkits.h"
#include "../../EnumDefinitions/EnumDefinitions.h"
#include "../../objects/objects.h"
#include "../../shared/shared.h"
#include "../../include/macros.h"
#include "../IoModel.h"

void CreateParametersPrognostic2(DataSet** pparameters,IoModel* iomodel,ConstDataHandle iomodel_handle){
	
	Param*   param = NULL;
	DataSet* parameters=NULL;
	int      count;
	int      i,j;
	int      dim;
	int*     part;

	double* vx_g=NULL;
	double* vy_g=NULL;
	double* h_g=NULL;
	double* a_g=NULL;
	double* m_g=NULL;

	/*recover parameters : */
	parameters=*pparameters;

	count=parameters->Size();

	/*Create transformation vector for DG inputs*/
	IoModelFetchData(&iomodel->elements,NULL,NULL,iomodel_handle,"elements");
	part=(int*)xcalloc(iomodel->numberofelements*3,sizeof(int));
	for(i=0;i<iomodel->numberofelements;i++){
		for(j=0;j<3;j++){
			part[3*i+j]=(int)iomodel->elements[3*i+j]-1; //Matlab to C indexing
			ISSMASSERT(part[3*i+j]<iomodel->numberofvertices);
		}
	}
	xfree((void**)&iomodel->elements);

	/*Get vx: */
	IoModelFetchData(&iomodel->vx,NULL,NULL,iomodel_handle,"vx");
	vx_g=(double*)xcalloc(iomodel->numberofelements*3,sizeof(double));
	if(iomodel->vx) for(i=0;i<3*iomodel->numberofelements;i++) vx_g[i]=iomodel->vx[part[i]]/iomodel->yts;

	count++;
	param= new Param(count,"vx_g",DOUBLEVEC);
	param->SetDoubleVec(vx_g,3*iomodel->numberofelements,1);
	parameters->AddObject(param);
	xfree((void**)&iomodel->vx);
	xfree((void**)&vx_g);

	/*Get vy: */
	IoModelFetchData(&iomodel->vy,NULL,NULL,iomodel_handle,"vy");
	vy_g=(double*)xcalloc(iomodel->numberofelements*3,sizeof(double));
	if(iomodel->vy) for(i=0;i<3*iomodel->numberofelements;i++) vy_g[i]=iomodel->vy[part[i]]/iomodel->yts;

	count++;
	param= new Param(count,"vy_g",DOUBLEVEC);
	param->SetDoubleVec(vy_g,3*iomodel->numberofelements,1);
	parameters->AddObject(param);
	xfree((void**)&iomodel->vy);
	xfree((void**)&vy_g);

	/*Get thickness: */
	IoModelFetchData(&iomodel->thickness,NULL,NULL,iomodel_handle,"thickness");
	h_g=(double*)xcalloc(iomodel->numberofelements*3,sizeof(double));
	if(iomodel->thickness) for(i=0;i<3*iomodel->numberofelements;i++) h_g[i]=iomodel->thickness[part[i]];

	count++;
	param= new Param(count,"h_g",DOUBLEVEC);
	param->SetDoubleVec(h_g,3*iomodel->numberofelements,1);
	parameters->AddObject(param);
	xfree((void**)&iomodel->thickness);
	xfree((void**)&h_g);

	/*Get accumulation: */
	IoModelFetchData(&iomodel->accumulation,NULL,NULL,iomodel_handle,"accumulation");
	a_g=(double*)xcalloc(iomodel->numberofelements*3,sizeof(double));
	if(iomodel->accumulation) for(i=0;i<3*iomodel->numberofelements;i++) a_g[i]=iomodel->accumulation[part[i]]/iomodel->yts;

	count++;
	param= new Param(count,"a_g",DOUBLEVEC);
	param->SetDoubleVec(a_g,3*iomodel->numberofelements,1);
	parameters->AddObject(param);
	xfree((void**)&iomodel->accumulation);
	xfree((void**)&a_g);

	/*Get melting: */
	IoModelFetchData(&iomodel->melting,NULL,NULL,iomodel_handle,"melting");
	m_g=(double*)xcalloc(iomodel->numberofelements*3,sizeof(double));
	if(iomodel->melting) for(i=0;i<3*iomodel->numberofelements;i++) m_g[i]=iomodel->melting[part[i]]/iomodel->yts;

	count++;
	param= new Param(count,"m_g",DOUBLEVEC);
	param->SetDoubleVec(m_g,3*iomodel->numberofelements,1);
	parameters->AddObject(param);
	xfree((void**)&iomodel->melting);
	xfree((void**)&m_g);

	/*Free partitioning vector*/
	xfree((void**)&part);

	/*Assign output pointer: */
	*pparameters=parameters;
}
